@@ -10,31 +10,21 @@ module Test.Tasty.CmdLine
 import Options.Applicative
 import Data.Monoid
 import Data.Proxy
+import Data.Foldable
 import System.Exit
 
-
 import Test.Tasty.Core
-import Test.Tasty.CoreOptions
 import Test.Tasty.Ingredients
 import Test.Tasty.Options
+import Test.Tasty.Options.Env
+import Test.Tasty.Runners.Reducers
 
 -- | Generate a command line parser from a list of option descriptions
 optionParser :: [OptionDescription] -> Parser OptionSet
-optionParser = foldr addOption (pure mempty) where
-  addOption :: OptionDescription -> Parser OptionSet -> Parser OptionSet
-  addOption (Option (Proxy :: Proxy v)) p =
-    setOption <$> (optionCLParser :: Parser v) <*> p
-
--- suiteOptions doesn't really belong here (since it's not CmdLine
--- specific), but I didn't want to create a new module just for it.
-
--- | All the options relevant for this test suite. This includes the
--- options for the test tree and ingredients, and the core options.
-suiteOptions :: [Ingredient] -> TestTree -> [OptionDescription]
-suiteOptions ins tree =
-  coreOptions ++
-  ingredientsOptions ins ++
-  treeOptions tree
+optionParser = getApp . foldMap toSet where
+  toSet :: OptionDescription -> Ap Parser OptionSet
+  toSet (Option (Proxy :: Proxy v)) = Ap $
+    (singleOption <$> (optionCLParser :: Parser v)) <|> pure mempty
 
 -- | The command line parser for the test suite
 suiteOptionParser :: [Ingredient] -> TestTree -> Parser OptionSet
@@ -44,12 +34,16 @@ suiteOptionParser ins tree = optionParser $ suiteOptions ins tree
 -- ingredient list
 defaultMainWithIngredients :: [Ingredient] -> TestTree -> IO ()
 defaultMainWithIngredients ins testTree = do
-  opts <- execParser $
+  cmdlineOpts <- execParser $
     info (helper <*> suiteOptionParser ins testTree)
     ( fullDesc <>
       header "Mmm... tasty test suite"
     )
 
+  envOpts <- suiteEnvOptions ins testTree
+
+  let opts = envOpts <> cmdlineOpts
+
   case tryIngredients ins opts testTree of
     Nothing ->
       putStrLn
