@@ -37,6 +37,9 @@ data Outcome
   | Failure FailureReason -- ^ test failed because of the 'FailureReason'
   deriving (Show, Generic)
 
+-- | Time in seconds. Used to measure how long the tests took to run.
+type Time = Double
+
 -- | A test result
 data Result = Result
   { resultOutcome :: Outcome
@@ -50,6 +53,8 @@ data Result = Result
     --
     -- For a failed test, 'resultDescription' should typically provide more
     -- information about the failure.
+  , resultTime :: Time
+    -- ^ How long it took to run the test, in seconds.
   }
 
 -- | 'True' for a passed test, 'False' for a failed one.
@@ -64,6 +69,7 @@ exceptionResult :: SomeException -> Result
 exceptionResult e = Result
   { resultOutcome = Failure $ TestThrewException e
   , resultDescription = "Exception: " ++ show e
+  , resultTime = 0
   }
 
 -- | Test progress information.
