@@ -9,7 +9,7 @@ module Test.Tasty.Ingredients.ConsoleReporter
 
 import Prelude hiding (fail)
 import Control.Monad.State hiding (fail)
-import Control.Monad.Reader hiding (fail)
+import Control.Monad.Reader hiding (fail,reader)
 import Control.Concurrent.STM
 import Control.Exception
 import Control.DeepSeq
@@ -21,11 +21,15 @@ import Test.Tasty.Options
 import Test.Tasty.Runners.Reducers
 import Text.Printf
 import qualified Data.IntMap as IntMap
+import Data.Char
 import Data.Maybe
 import Data.Monoid
 import Data.Proxy
+import Data.Tagged
 import Data.Typeable
 import Data.Foldable (foldMap)
+import Options.Applicative
+import Options.Applicative.Types (ReadM(..))
 import System.IO
 import System.Console.ANSI
 
@@ -72,9 +76,20 @@ produceOutput opts tree =
         printTestResult result = do
           rDesc <- formatMessage $ resultDescription result
 
+          -- use an appropriate printing function
+          let
+            printFn =
+              if resultSuccessful result
+                then ok
+                else fail
+            time = resultTime result
           if resultSuccessful result
-            then ok "OK\n"
-            else fail "FAIL\n"
+            then printFn "OK"
+            else printFn "FAIL"
+          -- print time only if it's significant
+          when (time >= 0.01) $
+            printFn (printf " (%.2fs)" time)
+          printFn "\n"
 
           when (not $ null rDesc) $
             (if resultSuccessful result then infoOk else infoFail) $
@@ -217,16 +232,16 @@ computeStatistics = getApp . foldMap (\var -> Ap $
   (\r -> Statistics 1 (if resultSuccessful r then 0 else 1))
     <$> getResultFromTVar var)
 
-printStatistics :: (?colors :: Bool) => Statistics -> IO ()
-printStatistics st = do
+printStatistics :: (?colors :: Bool) => Statistics -> Time -> IO ()
+printStatistics st time = do
   printf "\n"
 
   case statFailures st of
     0 -> do
-      ok $ printf "All %d tests passed\n" (statTotal st)
+      ok $ printf "All %d tests passed (%.2fs)\n" (statTotal st) time
 
     fs -> do
-      fail $ printf "%d out of %d tests failed\n" fs (statTotal st)
+      fail $ printf "%d out of %d tests failed (%.2fs)\n" fs (statTotal st) time
 
 data FailureStatus
   = Unknown
@@ -268,6 +283,7 @@ consoleTestReporter =
   TestReporter
     [ Option (Proxy :: Proxy Quiet)
     , Option (Proxy :: Proxy HideSuccesses)
+    , Option (Proxy :: Proxy UseColor)
     ] $
   \opts tree -> Just $ \smap ->
 
@@ -281,11 +297,14 @@ consoleTestReporter =
       hSetBuffering stdout NoBuffering
 
       let
-        ?colors = isTerm
-      let
+        whenColor = lookupOption opts
         Quiet quiet = lookupOption opts
         HideSuccesses hideSuccesses = lookupOption opts
 
+      let
+        ?colors = useColor whenColor isTerm
+
+      let
         output = produceOutput opts tree
 
       case () of { _
@@ -297,16 +316,17 @@ consoleTestReporter =
         | otherwise -> consoleOutput output smap
       }
 
-      if quiet
-        then do
-          fst <- failureStatus smap
-          return $ case fst of
-            OK -> True
-            _ -> False
-        else do
-          stats <- computeStatistics smap
-          printStatistics stats
-          return $ statFailures stats == 0
+      return $ \time ->
+        if quiet
+          then do
+            fst <- failureStatus smap
+            return $ case fst of
+              OK -> True
+              _ -> False
+          else do
+            stats <- computeStatistics smap
+            printStatistics stats time
+            return $ statFailures stats == 0
 
 -- | Do not print test results (see README for details)
 newtype Quiet = Quiet Bool
@@ -328,6 +348,46 @@ instance IsOption HideSuccesses where
   optionHelp = return "Do not print tests that passed successfully"
   optionCLParser = flagCLParser Nothing (HideSuccesses True)
 
+-- | When to use color on the output
+data UseColor
+  = Never | Always | Auto
+  deriving (Eq, Ord, Typeable)
+
+-- | Control color output
+instance IsOption UseColor where
+  defaultValue = Auto
+  parseValue = parseUseColor
+  optionName = return "color"
+  optionHelp = return "When to use colored output. Options are 'never', 'always' and 'auto' (default: 'auto')"
+  optionCLParser =
+    option parse
+      (  long name
+      <> help (untag (optionHelp :: Tagged UseColor String))
+      )
+    where
+      name = untag (optionName :: Tagged UseColor String)
+      parse =
+        ReadM .
+        maybe (Left (ErrorMsg $ "Could not parse " ++ name)) Right .
+        parseValue
+
+-- | @useColor when isTerm@ decides if colors should be used,
+--   where @isTerm@ denotes where @stdout@ is a terminal device.
+useColor :: UseColor -> Bool -> Bool
+useColor when isTerm =
+  case when of
+    Never  -> False
+    Always -> True
+    Auto   -> isTerm
+
+parseUseColor :: String -> Maybe UseColor
+parseUseColor s =
+  case map toLower s of
+    "never"  -> return Never
+    "always" -> return Always
+    "auto"   -> return Auto
+    _        -> Nothing
+
 -- }}}
 
 --------------------------------------------------
