@@ -1,5 +1,6 @@
 -- | Running tests
-{-# LANGUAGE ScopedTypeVariables, ExistentialQuantification, RankNTypes, FlexibleContexts #-}
+{-# LANGUAGE ScopedTypeVariables, ExistentialQuantification, RankNTypes,
+             FlexibleContexts, BangPatterns #-}
 module Test.Tasty.Run
   ( Status(..)
   , StatusMap
@@ -10,6 +11,7 @@ import qualified Data.IntMap as IntMap
 import qualified Data.Sequence as Seq
 import qualified Data.Foldable as F
 import Data.Maybe
+import Data.Time.Clock.POSIX
 import Control.Monad.State
 import Control.Monad.Writer
 import Control.Monad.Reader
@@ -19,6 +21,7 @@ import Control.Concurrent.Async
 import Control.Exception as E
 import Control.Applicative
 import Control.Arrow
+import GHC.Conc (labelThread)
 
 import Test.Tasty.Core
 import Test.Tasty.Parallel
@@ -92,8 +95,9 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
     -- If all initializers ran successfully, actually run the test.
     -- We run it in a separate thread, so that the test's exception
     -- handler doesn't interfere with our timeout.
-    withAsync (action yieldProgress) $ \asy ->
-      applyTimeout timeoutOpt $ wait asy
+    withAsync (action yieldProgress) $ \asy -> do
+      labelThread (asyncThreadId asy) "tasty_test_execution_thread"
+      timed $ applyTimeout timeoutOpt $ wait asy
 
   -- no matter what, try to run each finalizer
   mbExn <- destroyResources restore
@@ -101,7 +105,7 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
   atomically . writeTVar statusVar $ Done $
     case resultOrExn <* maybe (Right ()) Left mbExn of
       Left ex -> exceptionResult ex
-      Right r -> r
+      Right (t,r) -> r { resultTime = t }
 
   where
     initResources :: IO ()
@@ -136,6 +140,7 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
             { resultOutcome = Failure $ TestTimedOut t
             , resultDescription =
                 "Timed out after " ++ tstr
+            , resultTime = fromIntegral t
             }
       fromMaybe timeoutResult <$> timeout t a
 
@@ -234,25 +239,37 @@ getResource var =
       Destroyed -> throwSTM UseOutsideOfTest
       _ -> throwSTM $ unexpectedState "getResource" rState
 
--- | Start running all the tests in a test tree in parallel. The number of
--- threads is determined by the 'NumThreads' option.
---
--- Return a map from the test number (starting from 0) to its status
--- variable.
+-- | Start running all the tests in a test tree in parallel, without
+-- blocking the current thread. The number of test running threads is
+-- determined by the 'NumThreads' option.
 launchTestTree
   :: OptionSet
   -> TestTree
-  -> (StatusMap -> IO a)
+  -> (StatusMap -> IO (Time -> IO a))
+    -- ^ A callback. First, it receives the 'StatusMap' through which it
+    -- can observe the execution of tests in real time. Typically (but not
+    -- necessarily), it waits until all the tests are finished.
+    --
+    -- After this callback returns, the test-running threads (if any) are
+    -- terminated and all resources acquired by tests are released.
+    --
+    -- The callback must return another callback (of type @'Time' -> 'IO'
+    -- a@) which additionally can report and/or record the total time
+    -- taken by the test suite. This time includes the time taken to run
+    -- all resource initializers and finalizers, which is why it is more
+    -- accurate than what could be measured from inside the first callback.
   -> IO a
 launchTestTree opts tree k = do
   (testActions, rvars) <- createTestActions opts tree
   let NumThreads numTheads = lookupOption opts
-  abortTests <- runInParallel numTheads (fst <$> testActions)
-  (do let smap = IntMap.fromList $ zip [0..] (snd <$> testActions)
-      k smap)
-   `finally` do
-      abortTests
-      waitForResources rvars
+  (t,k) <- timed $ do
+     abortTests <- runInParallel numTheads (fst <$> testActions)
+     (do let smap = IntMap.fromList $ zip [0..] (snd <$> testActions)
+         k smap)
+      `finally` do
+         abortTests
+         waitForResources rvars
+  k t
   where
     alive :: Resource r -> Bool
     alive r = case r of
@@ -269,3 +286,15 @@ launchTestTree opts tree k = do
 
 unexpectedState :: String -> Resource r -> SomeException
 unexpectedState where_ r = toException $ UnexpectedState where_ (show r)
+
+-- | Measure the time taken by an 'IO' action to run
+timed :: IO a -> IO (Time, a)
+timed t = do
+  start <- getTime
+  !r    <- t
+  end   <- getTime
+  return (end-start, r)
+
+-- | Get system time
+getTime :: IO Time
+getTime = realToFrac <$> getPOSIXTime
